(function start() {
    browser.storage.local.get(['mappingList', 'baseUrl'], function(response) {
        var mappingList = response.mappingList;
        if (mappingList) {
            initCspHeaderListener(response);
        }
    });
})();

browser.runtime.onMessage.addListener(
    function(request, sender, sendResponse) {
        if (request.contentScriptQuery === "getConfigList") {
            var xhr = new XMLHttpRequest();
            xhr.onreadystatechange = function () {
                if (xhr.readyState === 4 && xhr.status === 200) {
                    var response = JSON.parse(xhr.responseText);
                    sendResponse(response);
                }
            };

            xhr.open("GET", request.url, true);
            xhr.send();

            return true;
        }
    }
);

browser.runtime.onMessage.addListener(
    function(request, sender, sendResponse) {
        if (request.contentScriptQuery === "getServerBaseUrl") {
            var xhr = new XMLHttpRequest();
            xhr.onreadystatechange = function () {
                if (xhr.readyState === 4 && xhr.status === 200) {
                    var response = JSON.parse(xhr.responseText);
                    sendResponse(response);
                }
            };
            xhr.open("GET", request.url, true);
            xhr.send();

            return true;
        }
    }
);

browser.runtime.onMessage.addListener(
    function(request, sender, sendResponse) {
        if (request.contentScriptQuery === "getSiteMappings") {
            var xhr = new XMLHttpRequest();
            xhr.onreadystatechange = function () {
                if (xhr.readyState === 4 && xhr.status === 200) {
                    var response = JSON.parse(xhr.responseText);
                    // init listener every time we get new mappings
                    browser.storage.local.get(['baseUrl', 'mappingList'], function (res) {
                        if (res.baseUrl) {
                            initCspHeaderListener({ mappingList: response.mappingList, baseUrl: res.baseUrl });
                        }
                    });
                    sendResponse(response);
                }
            };
            xhr.open("GET", request.url, true);
            xhr.send();

            return true;
        }
    }
);

var cspHeadersListener;

function removeCspHeadersListener() {
    if (cspHeadersListener) {
        browser.webRequest.onBeforeRequest.removeListener(cspHeadersListener);
    }
    cspHeadersListener = null;
}

function initCspHeaderListener(response) {
    var mappingList = response.mappingList;
    var baseUrl = response.baseUrl;

    var STRIPPED_WWW = 'www.';
    var STARTS_WITH = 'STARTS_WITH';
    var PAGE = 'PAGE';
    var DOMAIN = 'DOMAIN';

    function getStartsWithUrl(url) {
        if (url.indexOf('//') > -1) {
            return stripWWW(url.split('//')[1]);
        }
        return stripWWW(url);
    }

    function stripWWW (url) {
        if (url.substring(0, 4) === STRIPPED_WWW) {
            url = url.substring(STRIPPED_WWW.length);
        }
        return url
    }

    function findMatch(mappingList, pageUrl) {
        if (!mappingList) {
            return -1;
        }

        var startsWithUrl = getStartsWithUrl(pageUrl);
        var urlDomain = stripWWW(pageUrl);

        var hashedUrl;
        var bitArray;
        for (var i = 0; i < mappingList.length; i++) {
            var hashType = mappingList[i].hashType;
            switch (true) {
                case hashType.indexOf(STARTS_WITH) > -1:
                {
                    var splitted = hashType.split('_');
                    var urlLength = Number(splitted[splitted.length - 1]);
                    bitArray = sjcl.hash.sha256.hash(startsWithUrl.substring(0, urlLength));
                    hashedUrl = sjcl.codec.hex.fromBits(bitArray);
                    break;
                }
                case hashType.indexOf(PAGE) > -1:
                {
                    bitArray = sjcl.hash.sha256.hash(pageUrl);
                    hashedUrl = sjcl.codec.hex.fromBits(bitArray);
                    break;
                }
                case hashType.indexOf(DOMAIN) > -1:
                {
                    bitArray = sjcl.hash.sha256.hash(urlDomain);
                    hashedUrl = sjcl.codec.hex.fromBits(bitArray);
                    break;
                }
            }

            if (hashedUrl === mappingList[i].hash && mappingList[i].allowCSP) {
                console.log('Found match for given csp policy.');
                return i;
            }
        }

        return -1;
    }

    var onHeadersReceived = function(details) {
        var matchIndex = findMatch(mappingList, details.url);
        if (matchIndex === -1) {
            console.log('NO MATCH FOUND');
            return {
                responseHeaders: details.responseHeaders
            }
        }
        for (var i = 0; i < details.responseHeaders.length; i++) {
            if ('content-security-policy' === details.responseHeaders[i].name.toLowerCase().trim()) {
                var splitted = details.responseHeaders[i].value.split(';');
                for (var j = 0; j < splitted.length; j++) {
                    if (splitted[j].toLowerCase().indexOf('upgrade-insecure-requests') > -1) continue;

                    splitted[j] += " 'unsafe-eval' ";
                    // splitted[j] += " 'report-sample' "; // Uncomment for debug
                    var styleSrcIndex = splitted[j].toLowerCase().trim().indexOf('style-src');
                    var scriptSrcIndex = splitted[j].toLowerCase().trim().indexOf('script-src');

                    if (styleSrcIndex === 0) {
                        splitted[j] += " 'unsafe-inline' ";
                    }
                    if (scriptSrcIndex > -1) {
                        var nonceStart = splitted[j].indexOf("'nonce");
                        if (nonceStart > -1) {
                            var nonceEnd = splitted[j].substring(nonceStart+1).indexOf("'") + nonceStart+2;
                            splitted[j] = splitted[j].substring(0, nonceStart) + splitted[j].substring(nonceEnd);
                        }
                        splitted[j] += " 'unsafe-inline' ";
                    }

                    splitted[j] += baseUrl;
                }
                details.responseHeaders[i].value = splitted.join(';');
                // details.responseHeaders[i].value = ""; // IT REMOVES ALL DEFENSE! FOR DEBUGGING PURPOSES ONLY!
            }
        }

        return {
            responseHeaders: details.responseHeaders
        };
    };

    var filter = {
        urls: ["*://*/*"],
        types: ["main_frame", "sub_frame"]
    };

    if (cspHeadersListener) {
        removeCspHeadersListener();
    }

    browser.webRequest.onHeadersReceived.addListener(onHeadersReceived, filter, ["blocking", "responseHeaders"]);
    cspHeadersListener = onHeadersReceived;
}
